
<!DOCTYPE html>
<html lang="en">
  <head>

    {% block head %}
      <!-- Global site tag (gtag.js) - Google Analytics -->
      <script async src="https://www.googletagmanager.com/gtag/js?id=UA-118975934-1"></script>
      <script>
          window.dataLayer = window.dataLayer || [];

          function gtag() {
              dataLayer.push(arguments);
          }

          gtag('js', new Date());

          gtag('config', 'UA-118975934-1');
      </script>

      <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <!-- The above 3 meta tags *must* come first in the head; any other head content must come *after* these tags -->
    <meta name="description" content="">
    <meta name="author" content="">

    <title>Aequitas - The Bias Report</title>

    <!-- Bootstrap core CSS -->
    <link href="/static/css/bootstrap.min.css" rel="stylesheet">

    <link href="/static/css/styles.css" rel="stylesheet">

    <!-- IE10 viewport hack for Surface/desktop Windows 8 bug -->
    <link href="/static/assets/css/ie10-viewport-bug-workaround.css" rel="stylesheet">

    <!-- Custom styles for this template -->
      <link href="/static/css/jumbotron.css" rel="stylesheet">

    <!-- Just for debugging purposes. Don't actually copy these 2 lines! -->
    <!--[if lt IE 9]><script src="/static/assets/js/ie8-responsive-file-warning.js"></script><![endif]-->
    <script src="/static/assets/js/ie-emulation-modes-warning.js"></script>
    <script src="/static/js/toggle_radio.js"></script>

    <!-- HTML5 shim and Respond.js for IE8 support of HTML5 elements and media queries -->
    <!--[if lt IE 9]>
      <script src="https://oss.maxcdn.com/html5shiv/3.7.3/html5shiv.min.js"></script>
      <script src="https://oss.maxcdn.com/respond/1.4.2/respond.min.js"></script>
    <![endif]-->

      <link rel="stylesheet" type="text/css" href="//fonts.googleapis.com/css?family=Open+Sans"/>
      <style>
          body {
              font-family: 'Open Sans', sans-serif;
          }
      </style>
    {% endblock %}

  </head>



  <body>

  <div class="container">
      <!-- BEGIN HEADER -->
      <div class="header clearfix">
          <nav>
            <ul class="nav nav-pills pull-right">
              {% if request.path.find('/report-') > 0 %}
                <li role="presentation" class="active"><a href="{{go_back}}">Back to Customize</a><br></li>
                <li role="presentation"><a href="{{ url_for('home')}}">Home</a></li>
              {% else %}
                <li role="presentation" class="active"><a href="{{ url_for('home')}}">Home</a></li>
                <li role="presentation"><a href="http://github.com/dssg/aequitas" target="_blank">Code</a></li>
              {% endif %}

              <li role="presentation"><a href="http://dsapp.uchicago.edu/aequitas" target="_blank">About</a></li>
            </ul>
          </nav>

          <a href="{{ url_for('home')}}">
                <img alt="Brand" src="/static/images/aequitas_header.png" width="15%" class="img-fluid">
          </a>
      </div>

      <!-- END HEADER -->

      <div id="content">
        <h1 style="text-align:center; color: #b37d4e;"> {% block title %}{% endblock %} </h1>
        <h4 class="space-out">{% block description %}{% endblock %}</h4>
        <div class="space-out">{% block flow %}{% endblock %}</div>
          {% block content %}{% endblock %}
      </div>


    </div>


      <br>
      <br>
      <br>
      <br>
      <br>

      <footer align="center" class="footer">
          <p>&copy; 2018 Center for Data Science and Public Policy - University of Chicago</p>
      </footer>
    </div> <!-- /container -->


    <!-- IE10 viewport hack for Surface/desktop Windows 8 bug -->
    <script src="/static/assets/js/ie10-viewport-bug-workaround.js"></script>
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js"></script>
    <script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/js/bootstrap.min.js"></script>

    <script>
        $(document).ready(function () {
            $('[data-toggle="tooltip"]').tooltip();
        });
    </script>
  </body>

</html>
